//
// Created by webpigeon on 17/04/22.
//

#ifndef FGGL_GFX_OGL4_MODELS_HPP
#define FGGL_GFX_OGL4_MODELS_HPP

#include <string>
#include <unordered_map>

#include "fggl/gfx/ogl/backend.hpp"
#include "fggl/gfx/ogl/types.hpp"
#include "fggl/ecs3/ecs.hpp"

namespace fggl::gfx::ogl4 {

	struct StaticModel {
		constexpr static const char name[] = "StaticModel";

		std::shared_ptr<ogl::Shader> pipeline;
		std::shared_ptr<ogl::VertexArray> vao;

		// element data
		std::shared_ptr<ogl::ElementBuffer> elements;
		ogl::Primative drawType;
		std::size_t elementCount;
	};

	class StaticModelRenderer {
		public:
			StaticModelRenderer() = default;
			~StaticModelRenderer() = default;

			StaticModelRenderer(const StaticModelRenderer& other) = delete;
			StaticModelRenderer(StaticModelRenderer&& other) = delete;

			StaticModelRenderer& operator=(const StaticModelRenderer& other) = delete;
			StaticModelRenderer& operator=(StaticModelRenderer&& other) = delete;

			void render(ecs3::World& world) {
				resolveModels(world);
				renderModelsForward(world);
			}

		private:
			/**
			 * Attach any missing rendering components to models.
			 */
			void resolveModels(ecs3::World& world);

			/**
			 * Render all visible objects according to their render tokens.
			 */
			void renderModelsForward(const ecs3::World& world);

			ogl::VertexArray m_vao;
			ogl::ArrayBuffer m_vertexList;
			ogl::ElementBuffer m_indexList;
	};

}

#endif //FGGL_GFX_OGL4_MODELS_HPP
